#!/usr/bin/env python
# ldap.py 
#
# CONFIDENTIAL - SOURCE MATERIALS
#
# The code is a part of ProtoVer testsuite - use only under the license agreement
# specified in LICENSE.txt in your ProtoVer distribution
#
# Copyright 2005-2006, GLEG Ltd.


"""
LDAP runners
"""

import sys
import string
import time
import threading
import getopt
import socket

sys.path.append("libs")
sys.path.append("../libs")
sys.path.append("../../libs")

from utils import *

VERSION="1.2"

OPTIONS=[
{
        "name" : "host",
        "opt" : "target host",
        "type" : str,
        "default" : "127.0.0.1",
        "help" : "target host"
},
{
        "name" : "port",
        "opt" : "target port",
        "type" : int,
        "default" : "389",
        "help" : "target port"
},
{
        "name" : "timeout",
        "opt" : "timeout for network operations",
        "type" : float,
        "default" : "3.0",
        "help" : "timeout for IO operations"
},
{
        "name" : "start",
        "opt" : "start from this testcase (index >= 0)",
        "type" : int,
        "default" : "0",
        "help" : "start from this testcase"
},
{
        "name" : "num",
        "opt" : "max number of tests",
        "type" : int,
        "default" : "-1",
        "help" : "max number of tests"
},
{
        "name" : "read",
        "opt" : "always read server's response",
        "type" : int,
        "default" : "0",
        "help" : "always read server's response (default: off)"
},

{
        "name" : "delay",
        "opt" : "delay between tests",
        "type" : float,
        "default" : "0.1",
        "help" : "delay between tests (default: 0.1 sec)"
},
{
	"name" : "debug",
	"opt" : "debug",
	"type" : int,
	"default" : "0",
    	"help" :
                """debug level:
                   1 - print request and exit
                   2 - print server response and exit
                   3 - print all packets"""

}
]

class LDAPRunner(threading.Thread):
	def __init__(self, event, opts, logger, description="", version=""):
		threading.Thread.__init__(self)
		self.event = event
		self.logger = logger
		self.description = description
		self.version = version
		self.opts = opts
		self.status = "running"
		self.stop = 0
		self.last_packet = ""

	def finish(self):
		self.logger.write("Done\n")
		self.event.clear()

	def generate(self, f):
		pass

	def postconnect(self,f):
		pass

        def hexdump(self, s=""):
                i = 0
                tmp = ""
                for c in s:
                        i += 1
                        tmp += " %02x" % (ord(c),)
                        if (i%16)==0: tmp += "\n"
                return tmp



	def run(self):
      		self.logger.clear()
                self.logger.write("Running %s version %s\n" % (self.description, self.version))

		f=None

		#XXXX:only in demo versin
		n = 0
		for i in range(0,1):
				if self.stop == 1:
					return self.finish()

				self.event.wait()

  				if self.opts["num"] == 0:
                        		return self.finish()
                		
				if self.opts["num"] > 0:
                        		self.opts["num"] -= 1

				databuf = self.generate(None)

				if self.opts["start"] > 0 and n < self.opts["start"]:
					self.logger.write("skipping testcase %d\n"%n)
					n += 1
					continue 

				self.logger.write("%s: testcase %d\n"% (self.description,n))

  				if self.opts["debug"] == 1 or self.opts["debug"] == 3:
                                        self.logger.write("C->S:\n%s" % self.hexdump(databuf))

 				if self.opts["debug"] == 1:
                                        return self.finish()

				#XXXX:only in demo version
					
				sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
				try:
					sock.connect((self.opts["host"],self.opts["port"]))	
				except:

					self.logger.write("Error: failed to connect to %s:%d\n"%(self.opts["host"], self.opts["port"]))
					if n == 0:
						self.status = "finished (failed to connect)"
					else:
						self.status = "finished (BUG found!)"

					return self.finish()

				#XXXX:only in demo version
				sock.sendall(databuf)
	
				self.last_packet = databuf
				databuf = ""
				
				r = ""
				if self.opts["read"] == 1 or self.opts["debug"] >=2:
					r = sock.recv(100000)

				sock.close()

				if self.opts["debug"] >= 2:
					self.logger.write("S->C:\n%s\n" % self.hexdump(r))

				if self.opts["debug"] == 2:
					self.status = "finished (debug=2)"
					return self.finish()
			
				n += 1
				time.sleep(self.opts["delay"])

		self.status = "finished (all tests passed)"
		return self.finish()

	def usage():
        	print "\nUsage: %s [options]" % sys.argv[0]
        	print "-t host   target hostname"
        	print "-p port   target port"
        	print "-w sec    timeout for network IO operations (default: 3 seconds)"
        	print "-n num    max number of tests"
        	print "-i index  start from this testcase (index >= 0)"
		print "-r        always read server's response (default: off)"
        	print "-D sec    delay between tests (default: 0.1 sec)"
        	print "-d num    set debug level:"
        	print "          1 - print hexdump of request and exit"
        	print "          2 - print server response and exit"
        	print "          3 - print hexdump of all packets"
        	sys.exit()

	usage = staticmethod(usage)

 	def getopts(help=""):
		if len(sys.argv) < 3:
			LDAPRunner.usage()

        	opts = {}
        	opts["host"] = "localhost"
        	opts["port"] = 389
        	opts["timeout"] = 3.0
		opts["debug"] = 0
		opts["delay"] = 0.1
		opts["num"] = -1
		opts["start"] = 0
		opts["read"] = 0

        	try:
                	(options, args)=getopt.getopt(sys.argv[1:],"t:p:w:i:n:D:d:r")
        	except getopt.GetoptError:
                	LDAPRunner.usage()

        	for o, a in options:
                	if o == "-t":
                        	opts["host"] = a
                	if o == "-p":
                        	opts["port"] = int(a)
                	if o == "-w":
                        	opts["timeout"] = float(a)
                	if o == "-i":
                        	opts["start"] = int(a)
                	if o == "-n":
                        	opts["num"] = int(a)
                	if o == "-r":
                        	opts["read"] = 1
                	if o == "-D":
                        	opts["delay"] = float(a)
                	if o == "-d":
                        	opts["debug"] = int(a)

		return opts

	getopts = staticmethod(getopts)

