#!/usr/bin/env python
# commandlineinterface.py
#
# CONFIDENTIAL - SOURCE MATERIALS
#
# The code is a part of ProtoVer testsuite - use only under the license agreement
# specified in LICENSE.txt in your ProtoVer distribution
# 
# Copyright 2005-2006, GLEG Ltd.
#

import sys
import os
import stat
import getopt
import time
import threading

sys.path.append("libs")
sys.path.append("../libs")
sys.path.append("../../libs")

from utils import *

VERSION="1.1"
AUTHOR="GLEG Ltd."

g_testmodules = []
g_testgroups = []

def init_modules():
	oldpath = [p for p in sys.path]
	base_path = os.path.abspath("modules")
        st = os.stat(base_path)
        if not stat.S_ISDIR(st.st_mode):
        	print "%s is not a directory!" % base_path
                sys.exit()

        dirs = [d for d in os.listdir(base_path) if d!="." and d!=".."]
        dirs.sort()
        for d in dirs:
        	dir = "%s/%s" % (base_path,d)
                st = os.stat(dir)
                if not stat.S_ISDIR(st.st_mode):
                	continue

                files = [f[:-3] for f in os.listdir(dir) if len(f)>3 and f.find(".py")==len(f)-3 and f.find(".pyc")==-1]
                files.sort()

                sys.path.append(dir)

                testcases = [f for f in files if f.find("exploit")==-1]
                for file in testcases:
                	try:
                        	mod = __import__("%s"% (file), [], [], [])
                        except Exception,e:
                        	print "Error while importing %s" % file
                                print e
                                sys.exit()
                        g_testmodules.append(mod)
			descr = getattr(mod, "DESCRIPTION")
			g_testgroups.append(descr)
	
	 	sys.path = [p for p in oldpath]

def usage():
	print "Usage: %s [options]" % sys.argv[0]
	print "-t host   target hostname"
	print "-p port   target port"
	print "-g name   group of tests" 
	print "-w sec    timeout for network IO operations (default: 3.0 seconds)"
	print "-i index  start from this testcase (index >= 0)"
	print "-n num    max number of tests"	
	print "-r        always read server's response (default: off)"
	print "-D sec    delay between tests (default: 0.1 sec)"
	print "-d num    set debug level:"
	print "          1 - print hexdump of request and exit"
	print "          2 - print server response and exit"
	print "          3 - print hexdump of all packets"

	print "\nAvailable groups of tests:"
	for g in g_testgroups:
		print "%s" % g

	print ""

	sys.exit()


def main():
	print "Version %s of PROTOVER LDAP testsuite" % VERSION
	print "2005-2006 %s\n" % AUTHOR

	init_modules()

	if len(sys.argv) < 3:
		usage()

	opts = {}
	opts["testgroup"] = "all"
	opts["host"] = "localhost"
	opts["port"] = 389
	opts["timeout"] = 3.0
	opts["start"] = 0
	opts["num"] = -1
	opts["read"] = 0
	opts["debug"] = 0	
	opts["delay"] = 0.1

    	try:
        	(options, args)=getopt.getopt(sys.argv[1:],"t:p:w:g:i:n:D:d:r")
    	except getopt.GetoptError:
        	usage()

    	for o, a in options:
		if o == "-g":
			opts["testgroup"] = a
        	if o == "-t":
            		opts["host"] = a
        	if o == "-p":
            		opts["port"] = int(a)
		if o == "-w":
			opts["timeout"] = float(a)
		if o == "-i":
			opts["start"] = int(a)
		if o == "-n":
			opts["num"] = int(a)
		if o == "-r":
			opts["read"] = 1
		if o == "-D":
			opts["delay"] = float(a)
		if o == "-d":
			opts["debug"] = int(a)

	modules = []
	if opts["testgroup"]=="all":
		modules = g_testmodules
	elif opts["testgroup"] in g_testgroups:
		modules = []
		for mod in g_testmodules:
			if opts["testgroup"] == getattr(mod,"DESCRIPTION"):
				modules.append(mod)
				break
	else:
		print "Error: unknown group of tests \"%s\"" % opts["testgroup"]
		sys.exit()

	for mod in modules:
	 	event = threading.Event()
        	event.set()
		runner = getattr(mod, "Runner")(event, opts, TextLogger())
                runner.event.set()
		runner.run()
	
	return 0

if __name__=="__main__":
	sys.exit(main())
