#!/usr/bin/env python
# pygtkinterface.py
# CONFIDENTIAL - SOURCE MATERIALS
#
# The code is a part of ProtoVer testsuite - use only under the license agreement
# specified in LICENSE.txt in your ProtoVer distribution
#
# Copyright 2005-2006, GLEG Ltd.

import os
import stat
import sys
import string
import time
import threading
import pygtk
pygtk.require("2.0")
import gtk
import gobject

sys.path.append("libs")
sys.path.append("../libs")
sys.path.append("../../libs")

DESCRIPTION="ProtoVer LDAP testsuite"
RELEASE="1.3"
AUTHOR="GLEG Ltd."
HELP={}
HELP["Testcases"]="""
Miscellaneous protocol testcases.
To start new testing session, first select the testcase module, fill necessary options then press 'Start'.

'Start' - start new session.
'Stop' - stop the current session.
'Continue' - continue the current stopped session.
'Close' - close the current finished session.
"""
HELP["Exploits"]="""
Miscellaneous exploit modules.
Most of them has been found with this testsuite.
To run the exploit, first select the exploit module, fill necessary options the press 'Start'.
"""

class TextLogger:
	"""
	Simple wrapper around gtk.TextView()
	"""
	def __init__(self, tv):
		self.tv = tv

	def clear(self):
                self.tv.get_buffer().set_text("")

        def write(self, text):
		gtk.threads_enter()
		s = ""
		for c in list(text):
			if string.printable.find(c) != -1:
				s += c
		text = s
                buffer = self.tv.get_buffer()
		buffer.insert(buffer.get_end_iter(), text)
		self.tv.show()
		gtk.threads_leave()

class LoggerThread(threading.Thread):
	""" Logging thread """
	def __init__(self, lock, logger, message, clear=0):
		threading.Thread.__init__(self)
		self.lock = lock
		self.logger = logger
		self.message = message
		self.clear = clear

	def run(self):
		self.lock.acquire(1)
		if self.clear == 1:
			self.logger.clear()
		self.logger.write(self.message)
		self.lock.release()
		
class GUIEngine:
	def __init__(self):
		self.top = None
		self.table = None
		self.tree = None
		self.main_frame = None
		self.options = None
		self.info = None
		self.info_text = None
		self.help = None
		self.help_text = None
		self.frames = {}
		self.panes = {}
		self.menu_items = (
			("/Control", None, None, 0, "<Branch>"),
			("/Control/Start new session", None, self.handle_start, 0, None),
			("/Control/Stop the current session", None, self.handle_stop, 0, None),
			("/Control/Continue the current session", None, self.handle_continue, 0, None),
		#	("/Control/Close the current session", None, self.handle_close, 0, None),
			("/Control/Quit", None, self.destroy, 0, None),
			("/Help", None, None, 0, "<Branch>"),
			("/Help/About", None, self.handle_about, 0, None)
		)

		self.vbox = None # hold Options vbox

		self.lock = threading.Lock()

		self.protocols = []
		self.modules = []
		self.moduleID = 0
		self.stopped = 0
		self.opts = {}

		self.sessionID = -1
		self.sessions = []
		self.sessions_logger = None

		self.init_modules()

  	def init_modules(self):
		#sys.path.append("./modules")
		oldpath = [p for p in sys.path]

	 	base_path = os.path.abspath("modules")
                st = os.stat(base_path)
                if not stat.S_ISDIR(st.st_mode):
                        print "%s is not a directory!" % base_path
                        sys.exit()

                dirs = [d for d in os.listdir(base_path) if d!="." and d!=".."]
                dirs.sort()
                for d in dirs:
                        dir = "%s/%s" % (base_path,d)
                        st = os.stat(dir)
                        if not stat.S_ISDIR(st.st_mode):
                                continue

                        files = [f[:-3] for f in os.listdir(dir) if len(f)>3 and f.find(".py")==len(f)-3 and f.find(".pyc")==-1]
                        files.sort()

			start = len(self.modules)
			sys.path.append(dir)

			testcases = [f for f in files if f.find("exploit")==-1]
                        for file in testcases:
				try:
                                	mod = __import__("%s"% (file), [], [], [])
				except Exception,e:
					print "Error while importing %s" % file
					print e
					sys.exit()
					
                                self.modules.append(mod)
			
			exploits = [f for f in files if f.find("exploit")>=0]
			for file in exploits:
				mod = __import__("%s"%(file), [], [], [])
				self.modules.append(mod)

			sys.path = [p for p in oldpath]		
	
			# protocol description
			proto = {
				"name" : d, 
				"start" : start,
				"num testcases" : len(testcases),
				"num exploits" : len(exploits)
			}

			self.protocols.append(proto)
		

	def module_name(self, col, cell, model, iter):
		cell.set_property("text", model.get_value(iter, 0))

	def module_description(self, col, cell, model, iter):
	 	mod = self.modules_table[model.get_value(iter,0)]
		cell.set_property("text", getattr(mod, "DESCRIPTION"))

	def module_open(self, path, data):
		""" Change current module ID """
	
		model = self.treeview.get_model() 
		iter = model.get_iter(path)
		name = model.get_value(iter,0)
  
		if name == "Testcases" or name == "Exploits":
			self.moduleID = -1
			text = HELP[name]
			self.init_options()
			self.show_options()
		else:
			for p in self.protocols:
				if name == p["name"]:
					self.moduleID = -1	
					text = "%s protocol testsuite" % name
					self.init_options()
					self.show_options()
					self.help.get_buffer().set_text(text)	
					return True
				
			self.moduleID = int(name)
			mod = self.modules[self.moduleID]
			text = getattr(mod, "HELP")

			# handle module options
			text += "\nModule options:\n"
			options = getattr(mod, "OPTIONS")
			for opt in options:
				text += "\"%s\" - %s\n" % (opt["opt"],opt["help"])

			self.init_options()
			for opt in options:
		 		self.add_opt(opt["name"], opt["opt"], opt["default"])
			self.show_options()

               	self.help.get_buffer().set_text(text)
		return True

	def get_opt(self, name):
		buffer = self.opts[name].get_buffer()
		return buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())

	def set_opt(self, name, value):
		# M. S.: If there is not exist such an option in this session, skip it:
		if name in self.opts:
			buffer = self.opts[name].get_buffer()
			buffer.set_text(str(value))
	def handle_start(self, widget, data=None):
		""" Start new session """

		if self.moduleID == -1:
			return

		self.sessionID = len(self.sessions)
		
		mod = self.modules[self.moduleID]
		descr = getattr(mod, "DESCRIPTION")
                version = getattr(mod, "VERSION")

		opts = getattr(mod, "OPTIONS")
		mod_opts = {}	
		for opt in opts:
			mod_opts[opt["name"]] = opt

 		opts = {"session ID" : self.sessionID, "module ID" : self.moduleID}

		for name in self.opts.keys():
			if name == "session ID" or name == "module ID":
				continue
			type = mod_opts[name]["type"]
			opts[name] = type(self.get_opt(name))

		window = gtk.ScrolledWindow()
               	window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
               	window.show()

  		tv = gtk.TextView()
               	tv.set_wrap_mode(gtk.WRAP_WORD)
               	tv.set_editable(False)
               	tv.show()
		window.add(tv)

		str = "#%d %s:%d" % (self.moduleID,opts["host"], opts["port"])	
               	self.session_book.append_page(window, gtk.Label(str))
		self.session_book.set_current_page(len(self.sessions)+1)
	
		logger = TextLogger(tv)
		runner = getattr(mod, "Runner")(threading.Event(), opts, logger)
		runner.setDaemon(1)
		runner.event.set()
		runner.start()
	
		self.sessions.append({
			"descr"    : descr,
			"opts"     : opts, 
			"runner"   : runner,
			"stopped"  : 0,
			"logger"   : logger,
			"time"     : time.strftime("%d. %m. %H:%M:%S")
		})

		self.set_opt("session ID", self.sessionID)

		return True

	def handle_stop(self, widget, data=None):
		""" Stops the current session """
	
		if len(self.sessions) < 1:
			return

		if self.sessionID == -1:
			return

		session = self.sessions[self.sessionID]

		# already stopped
		if session["stopped"] == 1: 
			return

		if not session["runner"].isAlive():
			return

		session["stopped"] = 1
		session["runner"].event.clear()

		t = LoggerThread(self.lock, session["logger"], "Session stopped\n")
		t.setDaemon(1)
		t.start()
		#session["logger"].write("Session stopped\n")

	def handle_continue(self, widget, data=None):
		""" Continue the current session """
		
		if len(self.sessions) < 1:
			return

		if self.sessionID == -1:
			return

		session = self.sessions[self.sessionID]

		# there is nothing to continue
		if session["stopped"] == 0:
			return

		session["stopped"] = 0
		session["runner"].event.set()
		
		t = LoggerThread(self.lock, session["logger"], "Session continued\n")
		t.setDaemon(1)
		t.start()
		#session["logger"].write("Session continued\n")

	def handle_close(self, widget, data=None):
		""" Close the finished session"""
		if len(self.sessions) < 1:
			return
		
		if self.sessionID == -1:
			return

		session = self.sessions[self.sessionID]
		
		if session["stopped"] == 1:
			return

		# remove finished session
		self.lock.acquire(1)
		if not session["runner"].isAlive():
			index = self.sessionID + 1
		#	self.sessions.pop(self.sessionID)
			if len(self.sessions) > 0:
				for s in self.sessions:
					s["opts"]["session ID"] -= 1
			if self.sessionID >= 0:	
				self.sessionID -= 1
			self.sessions.pop(index - 1)
			self.session_book.remove_page(index + 1)
			self.session_book.set_current_page(0)
		#	self.sessions.pop(index - 1)
		self.lock.release()
	
	def handle_about(self, widget, data):
		about = "This is a %s version %s" % (DESCRIPTION, VERSION)
		self.about = gtk.MessageDialog(self.top, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, about)
		self.about.connect("response", self.destroy_about)
		self.about.show()

	def handle_switch_session(self, notebook, page, page_num):
		# list all sessions
		if page_num == 0:
			self.sessionID = -1
			self.set_opt("session ID", self.sessionID)

			if not self.sessions_logger:
				return True

		#	print self.sessions

			str = ""
			for i in range(0, len(self.sessions)):
				session = self.sessions[i]
		
				status = "[running]"
				if not session["runner"].isAlive():
					status = "[finished]"
				elif session["stopped"] == 1:
					status = "[stopped]"

				opts = session["opts"]
				i2 = opts["module ID"]
				str += "#%d %s %s:%d %s %s\n" % (i2,session["descr"],opts["host"],opts["port"],session["time"],status)
			
			t = LoggerThread(self.lock, self.sessions_logger, str, clear=1)
                        t.setDaemon(1)
                        t.start()

			return True

		if page_num - 1 >= len(self.sessions):
			return True
		
		# change current session ID
	
		self.sessionID = page_num - 1
		session = self.sessions[self.sessionID]
		opts = session["opts"]

		moduleID = opts[ "module ID" ];
		self.moduleID = moduleID
		mod = self.modules[self.moduleID]
		text = getattr(mod, "HELP")
		# handle module options
		text += "\nModule options:\n"
		options = getattr(mod, "OPTIONS")
		for opt in options:
			text += "\"%s\" - %s\n" % (opt["opt"],opt["help"])

		self.init_options()
		for opt in options:
	 		self.add_opt(opt["name"], opt["opt"], opt["default"])
		self.show_options()

		# show help
               	buffer = gtk.TextBuffer()
               	buffer.set_text(text)
               	self.help.set_buffer(buffer)

		#print opts
		for key in opts.keys():
			self.set_opt(key, opts[key])

		#print "SESSION ID = %d" % self.sessionID

	def destroy_about(self, dialog, respid):
		self.about.destroy()

	def destroy(self, widget, data=""):
		sys.exit()

	def create_menu_items(self):
		return
		"""
		agroup = gtk.AccelGroup()
		factory = gtk.ItemFactory(gtk.MenuBar, "<main>", agroup)
		factory.create_items(self.menu_items)
		self.top.add_accel_group(agroup)
		menubar = factory.get_widget("<main>")
		self.vbox.pack_start(menubar, False, True, 0)
		menubar.show()
		"""
	
	def create_modules_tree(self):
		treestore = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
		"""
		parent = treestore.append(None, ("Testcases", "RADIUS testcases"))
		for i in range(0, self.num_testcases):
			mod = self.modules[i]		
			name = "%d" % i	
			descr = getattr(mod, "DESCRIPTION")
			treestore.append(parent, (name , descr))

		parent = treestore.append(None, ("Exploits", "Exploit modules"))
		for i in range(self.num_testcases, len(self.modules)):
			mod = self.modules[i]
			name = "%d" % i
			descr = getattr(mod, "DESCRIPTION")
			treestore.append(parent, (name, descr))
		"""
		
		for p in self.protocols:
			n = p["name"]
			top = treestore.append(None, ("%s" %n, "%s testsuite" % n))
			parent = treestore.append(top, ("Testcases", "%s testcases" % n))
			start = p["start"]
			for i in range(0, p["num testcases"]):
				mod = self.modules[start + i] 
				name = "%d" % (start + i)
				try:
					descr = getattr(mod, "DESCRIPTION")
				except Exception,e:
					print "Error: %s" % str(mod)
					print e
					sys.exit()
				treestore.append(parent, (name, descr))
			start += p["num testcases"]
			parent = treestore.append(top, ("Exploits", "%s exploit modules" % n))
			for i in range(0, p["num exploits"]):
				mod = self.modules[start + i]
				name = "%d" % (start + i)
				descr = getattr(mod, "DESCRIPTION")
				treestore.append(parent, (name,descr))

		self.treeview = gtk.TreeView(treestore)
	
		col = gtk.TreeViewColumn("ID", gtk.CellRendererText(), text=0)
		self.treeview.append_column(col)
	
		col = gtk.TreeViewColumn("DESCRIPTION", gtk.CellRendererText(), text=1)
		self.treeview.append_column(col)

 		selection = self.treeview.get_selection()
                selection.set_mode(gtk.SELECTION_SINGLE)
                selection.set_select_function(self.module_open, None)

		self.treeview.show()

		self.tree.add(self.treeview)
		self.tree.show_all()

  	def add_opt(self, name, opt, value, editable=1):
		vbox = self.vbox

                table = gtk.Table(1,2,True)
                table.set_border_width(1)
                table.show()

                label = gtk.Label(opt)
                label.set_alignment(0.0, 0.5)
                label.show()
                table.attach(label, 0, 1, 0, 1)

                tv = gtk.TextView()
                if editable == 0:
                        tv.set_editable(False)

                b = gtk.TextBuffer()
                b.set_text(value)
                tv.set_buffer(b)
                tv.show()
                frame= gtk.Frame()
                frame.add(tv)
                frame.show()
                frame.set_border_width(1)
                table.attach(frame, 1, 2, 0, 1)
                
		vbox.pack_start(table, True, False, 0)

                self.opts[name] = tv


	def init_options(self):
		""" Init Options box, add necessary options """

		self.opts = {}

		self.vbox = gtk.VBox(False, 0)
                self.vbox.set_border_width(0)

		self.add_opt("session ID", "session ID", "-1", editable=0)
		self.add_opt("module ID", "module ID", "%d" % self.moduleID, editable=0)

	def show_options(self):
		vbox = self.vbox

		hbox = gtk.HBox()
		hbox.set_border_width(0)

                b = gtk.Button("Start")
                b.connect("clicked", self.handle_start, None)
                b.show()
                hbox.pack_start(b,False, False,0)

                b = gtk.Button("Stop")
                b.connect("clicked", self.handle_stop, None)
                b.show()
                hbox.pack_start(b,False, False,0)

                b = gtk.Button("Continue")
                b.connect("clicked", self.handle_continue, None)
                b.show()
                hbox.pack_start(b, False, False, 0)

              #  b = gtk.Button("Close")
               # b.connect("clicked", self.handle_close, None)
               # b.show()
               # hbox.pack_start(b, False, False, 0)

                hbox.show()
                vbox.pack_start(hbox, False, False, 10)

		vbox.show()

		self.optbook.remove_page(-1)
  		self.optbook.append_page(vbox, gtk.Label("Options"))

	def create(self):
		self.top = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.top.set_title("%s v%s" % (DESCRIPTION, RELEASE))
		self.top.connect("destroy", self.destroy, "Exiting...")
		self.top.set_size_request(800,500)
		self.top.set_border_width(0)

		main_frame = gtk.Frame()
		main_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.top.add(main_frame)
		main_frame.show()

		self.vpaned = gtk.VPaned()
		main_frame.add(self.vpaned)
		self.vpaned.show()
		self.create_menu_items()


		hpaned = gtk.HPaned()
		self.vpaned.pack1(hpaned, True, False)
		hpaned.show()

		self.tree = gtk.ScrolledWindow()
		self.tree.set_border_width(0)
		self.tree.set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
		self.tree.set_size_request(300,200)
		self.create_modules_tree()
		hpaned.pack1(self.tree, True, False)
	
		vpaned2 = gtk.VPaned()
		hpaned.pack2(vpaned2, True, False)
		vpaned2.show()
		
		window = gtk.ScrolledWindow()
               	window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
		window.show()
		vpaned2.pack1(window, True, False)

		self.help = gtk.TextView()
		self.help.set_wrap_mode(gtk.WRAP_WORD)
		self.help.set_editable(False)
		self.help.show()
		window.add(self.help)

		self.optbook = gtk.Notebook()
		self.optbook.set_border_width(0)
		self.optbook.set_show_border(True)
                self.optbook.set_tab_pos(gtk.POS_TOP)
                self.optbook.set_scrollable(True)

                self.init_options()
		self.show_options()

		self.optbook.show()
		f = gtk.Frame()
		f.set_shadow_type(gtk.SHADOW_NONE)
		f.set_border_width(0)
		f.add(self.optbook)	

		vpaned2.pack2(f, True, False)
		f.show()
		
		self.info = gtk.Frame()	
		window = gtk.ScrolledWindow()
		window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
		window.show()
		self.vpaned.pack2(self.info, True, False)
		self.info.show()

		self.session_book = gtk.Notebook()
		self.session_book.set_tab_pos(gtk.POS_TOP)
		self.session_book.set_scrollable(True)
		self.session_book.connect("switch-page", self.handle_switch_session)
		self.session_book.show()

		tv = gtk.TextView()
		tv.set_wrap_mode(gtk.WRAP_WORD)
		tv.set_editable(False)
		tv.show()
		tv.set_border_width(0)
		window.add(tv)
		self.session_book.append_page(window, gtk.Label("Sessions"))

		self.sessions_logger = TextLogger(tv)
	
		self.info.add(self.session_book)

		#self.top.maximize()
		self.top.show()
		
	def main_nt(self):
		self.create()
		while gtk.main_iteration(False):
	           time.sleep(0.001 )

	def main(self):
		gtk.threads_init()
		gtk.threads_enter()
		self.create()
		gtk.threads_leave()
		gtk.main()

if __name__ == "__main__":
	print "Running %s v%s" % (DESCRIPTION, RELEASE)
	gui = GUIEngine()
	if os.name == "nt":
		gui.main_nt()
	else:
		gui.main()


